#!/usr/bin/Rscript

## NOTE
# XXX

## ARGS
ARGS    <- commandArgs(trailingOnly=TRUE)
COV_G   <- ARGS[1]
COV_T   <- ARGS[2]
UNIQ    <- ARGS[3]
GENES   <- ARGS[4]
TITLE   <- ARGS[5]
PDF     <- ARGS[6]
cols_cov <- c("contig", "start", "end", "coverage")
cols_gen <- c("gene", "start", "end", "strand", "info")

## IMPORT
suppressMessages(library(ggplot2)) # plotting
suppressMessages(library(reshape2)) # reshaping dataframes
suppressMessages(library(scales)) # plot scales
suppressMessages(library(viridis)) # color palette

## DATA
# Coverage
df_cov_G <- read.csv(file=COV_G, sep="\t", header=FALSE, stringsAsFactors=FALSE, col.names=cols_cov)
df_cov_T <- read.csv(file=COV_T, sep="\t", header=FALSE, stringsAsFactors=FALSE, col.names=cols_cov)
# Genes
df_uniq         <- read.csv(file=UNIQ,  sep="\t", header=FALSE, stringsAsFactors=FALSE, col.names=cols_gen)
df_genes        <- read.csv(file=GENES, sep="\t", header=FALSE, stringsAsFactors=FALSE, col.names=cols_gen)
df_genes$uniq   <- df_genes$gene %in% df_uniq$gene

## PLOT
cov_plot <- function(df_c, df_g){
    max_y <- max(df_c$coverage)
    pp <-
        ggplot(data=df_c) +
        geom_rect(data=df_g, aes(xmin=start, xmax=end, ymin=0, ymax=max_y, fill=uniq, colour=uniq), alpha=0.3) +
        geom_rect(aes(xmin=start, xmax=end, ymin=0, ymax=coverage), fill="black", colour=NA, alpha=1) +
        geom_hline(yintercept=10, colour="red") +
        scale_y_log10(breaks = trans_breaks("log10", function(x) 10^x), labels = trans_format("log10", math_format(10^.x))) +
        labs(
            title=TITLE,
            x="base",
            y="coverage"
        ) +
        theme_bw()
    return(pp)
}

pp_G <- cov_plot(df_cov_G, df_genes)
pp_T <- cov_plot(df_cov_T, df_genes)

pdf(PDF, width=60, height=4)
print(pp_G)
print(pp_T)
dev.off()
