#!/usr/bin/env python

# python diamond_annot_clusters.py /scratch/users/vgalata/GDB/results/annotation/diamond/sr/megahit/proteins.filtered.annot.tsv
# python diamond_annot_clusters.py /scratch/users/vgalata/GDB/results/annotation/diamond/lr/flye/proteins.filtered.annot.tsv

import sys
import pandas

if __name__ == "__main__":
    qids = pandas.read_csv(sys.argv[1], sep="\t", header=0, usecols=["qseqid", "sseqid"])
    # qids = pandas.read_csv("/scratch/users/vgalata/GDB/results/annotation/diamond/lr/flye/proteins.filtered.annot.tsv", sep="\t", header=0, usecols=["qseqid", "sseqid"])
    qids.sort_values(by=["qseqid"], axis=0, inplace=True)
    qids.set_index(keys="qseqid", drop=False, inplace=True, verify_integrity=True)
    qids["cluster_label"] = 0
    qids["cluster"] = False

    last_qid = ""
    last_cid = ""
    last_sid = -1
    cluster  = 0
    smatters = bool(sys.argv[2])
    for qid in qids.qseqid:
        qid_sid = int(qid.split("_")[-1])
        qid_cid = "_".join(qid.split("_")[:-1])
        # same contig, consecutive genes, (same subject or it does not matter)
        if last_cid == qid_cid and last_sid + 1 == qid_sid and (qids.loc[last_qid, "sseqid"] == qids.loc[qid, "sseqid"] or not smatters):
            qids.loc[last_qid, "cluster"] = True
            qids.loc[qid, "cluster"] = True
        else:
            cluster += 1
        qids.loc[qid, "cluster_label"] = cluster
        last_qid = qid
        last_cid = qid_cid
        last_sid = qid_sid

    print("{}:\nlooking for consecutive genes (clusters) where {}\n{} total hits\n{} ({:.2f}%) queries are in clusters\n{} clusters w/ average length of {:.2f} genes".format(
        sys.argv[1],
        "subject ID does not matter" if not smatters else "subject ID does matter",
        qids.shape[0],
        qids.cluster.sum(),
        100 * qids.cluster.sum() / qids.shape[0],
        len(set(qids.loc[qids.cluster,"cluster_label"])),
        qids.loc[qids.cluster,:].groupby("cluster_label")["sseqid"].agg(["size"]).mean().values[0]
    ))