#!/usr/bin/Rscript

# NOTE: code to be executed in report_render.R to generate data for report.Rmd

## DATA

##############################
# NanoStats
TABS$nanostats <- read_nanostats(snakemake@input$nanostats)

##############################
# Mappability

TABS$mappability <- list()
TABS$mappability$metag <- read_mappability(snakemake@input$mappability[1])
TABS$mappability$metag$tool <- factor(TABS$mappability$metag$tool, ordered=TRUE, levels=ASM_TOOL_NAMES)
if(length(snakemake@input$mappability) > 1){
    TABS$mappability$metat <- read_mappability(snakemake@input$mappability[2])
    TABS$mappability$metat$tool <- factor(TABS$mappability$metat$tool, ordered=TRUE, levels=ASM_TOOL_NAMES)
} else {
    TABS$mappability$metat <- NULL
}

FIGS$mappability <- list()
FIGS$mappability$metag <-
    ggplot(data=TABS$mappability$metag[TABS$mappability$metag$variable=="mapped_pct",], aes(x=tool, y=value, fill=tool)) +
    geom_col() +
    geom_text(aes(label=value, y=0.5*value), color="black", size=4, angle=90) +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    facet_wrap(vars(minlength), ncol=2, scales="fixed") +
    labs(
        x="",
        y="Mapped reads [%]"
    ) +
    mappability_theme

if(!is.null(TABS$mappability$metat)){
    FIGS$mappability$metat <-
        ggplot(data=TABS$mappability$metat[TABS$mappability$metat$variable=="mapped_pct",], aes(x=tool, y=value, fill=tool)) +
        geom_col() +
        geom_text(aes(label=value, y=0.5*value), color="black", size=4, angle=90) +
        scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
        facet_wrap(vars(minlength), ncol=2, scales="fixed") +
        labs(
            x="",
            y="Mapped reads [%]"
        ) +
        mappability_theme
}

##############################
# CRISPR

if("casc" %in% snakemake@config$steps_annotation){
    TABS$casc   <- read_crispr(snakemake@input$casc[1])
} else {
    TABS$casc   <- NULL
}
if("minced" %in% snakemake@config$steps_annotation){
    TABS$minced <- read_crispr(snakemake@input$minced[1])
} else {
    TABS$minced <- NULL
}
if(!is.null(TABS$casc) & !is.null(TABS$minced)){
    TABS$crispr <- rbind(TABS$casc, TABS$minced)
} else if(!is.null(TABS$casc)){
    TABS$crispr <- TABS$casc
} else if(!is.null(TABS$minced)){
    TABS$crispr <- TABS$minced
} else {
    TABS$crispr <- NULL
}
if(!is.null(TABS$crispr)){
    # aggregate: spacers per assembly
    TABS$crispr_aggr_sa <- aggregate(
        x=TABS$crispr$spacers,
        by=list(crispr_tool=TABS$crispr$crispr_tool, asm_tool=TABS$crispr$asm_tool),
        FUN=sum
    )
    TABS$crispr_aggr_sa$asm_tool <- factor(TABS$crispr_aggr_sa$asm_tool, ordered=TRUE, levels=ASM_TOOL_NAMES)
    # aggregate: arrays per assembly
    TABS$crispr_aggr_aa <- aggregate(
        x=TABS$crispr$seq_id,
        by=list(crispr_tool=TABS$crispr$crispr_tool, asm_tool=TABS$crispr$asm_tool),
        FUN=length
    )
    TABS$crispr_aggr_aa$asm_tool <- factor(TABS$crispr_aggr_aa$asm_tool, ordered=TRUE, levels=ASM_TOOL_NAMES)
}

if(!is.null(TABS$crispr)){
    FIGS$crispr <- list()
    if(length(unique(TABS$crispr_aggr_aa$crispr_tool)) > 1){
        FIGS$crispr$spacers <-
            ggplot(data=TABS$crispr_aggr_sa, aes(x=crispr_tool, y=x, fill=asm_tool)) +
            geom_col(position="dodge") +
            scale_fill_manual(values=ASM_TOOL_COLORS, guide="legend") +
            labs(
                x="",
                y="Number of spacers"
            ) +
            crispr_theme
        FIGS$crispr$arrays <-
            ggplot(data=TABS$crispr_aggr_aa, aes(x=crispr_tool, y=x, fill=asm_tool)) +
            geom_col(position="dodge") +
            scale_fill_manual(values=ASM_TOOL_COLORS, guide="legend") +
            labs(
                x="",
                y="Number of CRISPR arrays"
            ) +
            crispr_theme
    } else {
        FIGS$crispr$spacers <-
            ggplot(data=TABS$crispr_aggr_sa, aes(x=asm_tool, y=x, fill=asm_tool)) +
            geom_col(position="dodge") +
            scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
            labs(
                title=unique(TABS$crispr_aggr_aa$crispr_tool),
                x="",
                y="Number of spacers"
            ) +
            crispr_theme
        FIGS$crispr$arrays <-
            ggplot(data=TABS$crispr_aggr_aa, aes(x=asm_tool, y=x, fill=asm_tool)) +
            geom_col(position="dodge") +
            scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
            labs(
                title=unique(TABS$crispr_aggr_aa$crispr_tool),
                x="",
                y="Number of CRISPR arrays"
            ) +
            crispr_theme
    }
} else {
    FIGS$crispr <- NULL
}

##############################
# Plasflow: plasmid prediction

if("plasflow" %in% snakemake@config$steps_annotation){
    TABS$plasflow <- read_plasflow(snakemake@input$plasflow[1])
    TABS$plasflow_melted <- reshape2::melt(TABS$plasflow, id.vars=c("label", "type", "tool"), variable.name="statstype")
    TABS$plasflow_melted$tool <- factor(TABS$plasflow_melted$tool, ordered=TRUE, levels=ASM_TOOL_NAMES)
    FIGS$plasflow <- list()

    for(stype in c("sum_pct")){
        FIGS$plasflow[[stype]] <-
            ggplot(
                data=TABS$plasflow_melted[TABS$plasflow_melted$statstype == stype,],
                aes_string(x="tool", y="value", fill="label")
            ) +
            geom_col(position="dodge") +
            scale_fill_manual(values=PLASFLOW_COLORS$labels, guide="legend") +
            labs(
                x="",
                y=PLASFLOW_NAMES$statstype[stype]
            ) +
            plasflow_theme
    }
} else {
    TABS$plasflow <- NULL
    FIGS$plasflow <- NULL
}

##############################
# Prodigal

TABS$prodigal_gcounts <- read_prodigal_gcounts(snakemake@input$prodigal_gcounts)
TABS$prodigal_gcounts_melted <- reshape2::melt(TABS$prodigal_gcounts[,c("tool", "total", "partial")], id.vars=c("tool"))
TABS$prodigal_gcounts$tool        <- factor(TABS$prodigal_gcounts$tool,        ordered=TRUE, levels=ASM_TOOL_NAMES)
TABS$prodigal_gcounts_melted$tool <- factor(TABS$prodigal_gcounts_melted$tool, ordered=TRUE, levels=ASM_TOOL_NAMES)

TABS$prodigal_glength <- read_prodigal_glength(snakemake@input$prodigal_glength)
TABS$prodigal_glength$tool <- factor(TABS$prodigal_glength$tool, ordered=TRUE, levels=ASM_TOOL_NAMES)

FIGS$prodigal <- list()
FIGS$prodigal$gcounts <-
    ggplot(data=TABS$prodigal_gcounts_melted, aes(x=variable, y=value, fill=tool)) +
    geom_col(position="dodge") +
    scale_fill_manual(values=ASM_TOOL_COLORS) +
    labs(
        x="",
        y="Gene count"
    ) +
    prodigal_theme

FIGS$prodigal$gpct <-
    ggplot(data=TABS$prodigal_gcounts, aes(x=tool, y=partial_pct, fill=tool)) +
    geom_col() +
    scale_fill_manual(values=ASM_TOOL_COLORS) +
    labs(
        x="",
        y="Percentage of partial genes"
    ) +
    prodigal_theme

FIGS$prodigal$glength_violin <-
    ggplot(data=TABS$prodigal_glength, aes(x=tool, y=gene_length, fill=tool)) +
    geom_violin(draw_quantiles = c(0.25, 0.5, 0.75)) +
    scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    labs(
        x="",
        y="Gene length [bp]"
    ) +
    diamond_theme
FIGS$prodigal$glength_violin_zoom <- FIGS$prodigal$glength_violin + coord_cartesian(ylim=c(0, 2000))

# FIGS$prodigal$glength_density <-
#     ggplot(data=TABS$prodigal_glength, aes(x=gene_length, colour=tool, fill=tool)) +
#     geom_density(alpha=0.1) +
#     scale_colour_manual(values=ASM_TOOL_COLORS, guide=NULL) +
#     scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
#     facet_wrap(vars(tool), ncol=1, scales="fixed") +
#     labs(
#         x="Gene length [bp]",
#         y="Density"
#     ) +
#     diamond_theme

##############################
# DIAMOND

if("diamond" %in% snakemake@config$steps_annotation){
    TABS$diamond             <- read_diamond(snakemake@input$diamond[1])
    TABS$diamond_melted      <- reshape2::melt(TABS$diamond, id.vars=c("tool"))
    TABS$diamond$tool        <- factor(TABS$diamond$tool,        ordered=TRUE, levels=ASM_TOOL_NAMES)
    TABS$diamond_melted$tool <- factor(TABS$diamond_melted$tool, ordered=TRUE, levels=ASM_TOOL_NAMES)

    FIGS$diamond <- list()
    # FIGS$diamond$violin <-
    #     ggplot(data=TABS$diamond_melted, aes(x=tool, y=value, fill=tool)) +
    #     geom_violin(draw_quantiles = c(0.25, 0.5, 0.75)) +
    #     scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    #     facet_wrap(vars(variable), ncol=1, scales="fixed", labeller=labeller(variable=DIAMOND_VAR_LABELLER)) +
    #     labs(
    #         x="",
    #         y="Query/subject length ratio"
    #     ) +
    #     diamond_theme
    # FIGS$diamond$violin_zoom <- FIGS$diamond$violin + coord_cartesian(ylim=c(0, 1.5))
    # FIGS$diamond$density <-
    #     ggplot(data=TABS$diamond, aes(x=qs_len_ratio, colour=tool, fill=tool)) +
    #     geom_density(alpha=0.1) +
    #     scale_colour_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    #     scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
    #     facet_wrap(vars(tool), ncol=1, scales="free_y") +
    #     labs(
    #         x="Query/subject length ratio",
    #         y="Density"
    #     ) +
    #     diamond_theme
    for(col in unique(TABS$diamond_melted$variable)){
        FIGS$diamond[[sprintf("density_%s", col)]] <- plot_diamond_density(TABS$diamond, col)
    }
} else {
    TABS$diamond <- NULL
    FIGS$diamond <- NULL
}

##############################
# RGI: AMR prediction

if("rgi" %in% snakemake@config$steps_annotation){
    TABS$rgi        <- read_rgi(snakemake@input$rgi[1])
    TABS$rgi_melted <- reshape2::melt(TABS$rgi, id.vars=c("label", "col", "type"), variable.name="tool", value.name="count")
    TABS$rgi_aggr   <- TABS$rgi_melted[TABS$rgi_melted$col == "ARO",]
    TABS$rgi_aggr   <- aggregate(TABS$rgi_aggr$count, by=list(tool=TABS$rgi_aggr$tool, type=TABS$rgi_aggr$type), FUN=sum)
    TABS$rgi_aggr$tool <- factor(TABS$rgi_aggr$tool, levels=ASM_TOOL_NAMES, ordered=TRUE)

    FIGS$rgi  <- list()
    for(ctype in unique(TABS$rgi_melted$type)){
        # Counts per feature type
        for(col in c("Resistance mechanism")){
            df <- TABS$rgi_melted[TABS$rgi_melted$type == ctype & TABS$rgi_melted$col == col,]
            FIGS$rgi[[sprintf("%s_%s", ctype, col)]] <-
                ggplot(data=df, aes(x=tool, y=count, fill=tool)) +
                geom_col() +
                scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
                facet_wrap(vars(label), ncol=round(sqrt(length(unique(df$label)))), scales="free_y") +
                labs(
                    x="",
                    y=sprintf("%s (%s hits)", col, ctype)
                ) +
                rgi_theme
        }
    }
    # Total counts
    FIGS$rgi[[sprintf("total")]] <-
        ggplot(data=TABS$rgi_aggr, aes(x=tool, y=x, fill=tool)) +
        geom_col() +
        scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
        facet_wrap(vars(type), ncol=3) +
        labs(
            x="",
            y="Total hits"
        ) +
        rgi_theme
} else {
    TABS$rgi <- NULL
    FIGS$rgi <- NULL
}

##############################
# barrnap: rRNA gene prediction

if("barrnap" %in% snakemake@config$steps_annotation){
    TABS$barrnap       <- read_barrnap(snakemake@input$barrnap[1])
    # total counts per tool, kingdom and partial/complete categories
    TABS$barrnap_total <- aggregate(
        TABS$barrnap$count,
        by=list(tool=TABS$barrnap$tool, kingdom=TABS$barrnap$kingdom, partial=grepl("partial", TABS$barrnap$gene)),
        FUN=sum
    )
    # partial: FALSE/TRUE -> label
    TABS$barrnap_total$partial <- c("complete", "partial")[TABS$barrnap_total$partial + 1]
    # add partial + complete = total
    TABS$barrnap_total <- rbind(
        TABS$barrnap_total,
        aggregate(
            TABS$barrnap_total$x,
            by=list(tool=TABS$barrnap_total$tool, kingdom=TABS$barrnap_total$kingdom, partial=rep("total", nrow(TABS$barrnap_total))),
            FUN=sum
        )
    )

    TABS$barrnap$tool       <- factor(TABS$barrnap$tool, ordered=TRUE, levels=ASM_TOOL_NAMES)
    TABS$barrnap_total$tool <- factor(TABS$barrnap_total$tool, ordered=TRUE, levels=ASM_TOOL_NAMES)

    FIGS$barrnap <- list()
    FIGS$barrnap$total <-
        ggplot(data=TABS$barrnap_total, aes(x=tool, y=x, fill=tool)) +
        geom_col() +
        scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
        facet_grid(vars(partial), vars(kingdom), scales="fixed") +
        labs(
            x="",
            y=sprintf("rRNA gene hits")
        ) +
        barrnap_theme
    for(kingdom in sort(unique(TABS$barrnap$kingdom))){
        FIGS$barrnap[[sprintf("genes_%s", kingdom)]] <-
            ggplot(data=TABS$barrnap[TABS$barrnap$kingdom == kingdom,], aes(x=tool, y=count, fill=tool)) +
            geom_col() +
            scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
            facet_wrap(vars(gene), ncol=1, scales="free_y") +
            labs(
                x="",
                y=sprintf("rRNA gene hits (%s)", kingdom)
            ) +
            barrnap_theme
    }
} else {
    TABS$barrnap <- NULL
    FIGS$barrnap <- NULL
}

##############################
# QAUST: assembly quality

if("quast" %in% snakemake@config$steps_analysis){
    TABS$quast <- read_quast(snakemake@input$quast[1])
    TABS$quast_melted <- reshape2::melt(
        cbind(stat_vars=rownames(TABS$quast), TABS$quast),
        id.vars="stat_vars"
    )
    TABS$quast_melted$variable <- factor(TABS$quast_melted$variable, ordered=TRUE, levels=ASM_TOOL_NAMES)

    FIGS$quast <-
        ggplot(data=TABS$quast_melted, aes(x=variable, y=value)) +
        geom_col(aes(fill=variable)) +
        scale_fill_manual(values=ASM_TOOL_COLORS, guide=NULL) +
        facet_wrap(vars(stat_vars), ncol=2, scales="free_y") +
        labs(
            x="",
            y="QUAST statistic"
        ) +
        quast_theme
} else {
    TABS$quast <- NULL
    FIGS$quast <- NULL
}

##############################
# Mash

if("mash_dist" %in% snakemake@config$steps_analysis){
    TABS$mash <- list(
        reads=read_mash_dist_reads(snakemake@input$mash_reads[1]),
        asm=read_mash_dist_asm(snakemake@input$mash_asm[1])
    )

    mash_reads_htree <- hclust(d=as.dist(TABS$mash$reads), method="complete")
    mash_asm_htree   <- hclust(d=as.dist(TABS$mash$asm),    method="complete")

    FIGS$mash <- list()
    FIGS$mash$reads <- pheatmap(
        TABS$mash$reads,
        color=viridis(20),
        border_color="white",
        scale="none",
        cluster_rows=mash_reads_htree,
        cluster_cols=mash_reads_htree,
        display_numbers=TRUE, number_color="white", number_format="%.3f", fontsize_number=10,
        silent=TRUE
    )
    FIGS$mash$asm <- pheatmap(
        TABS$mash$asm,
        color=viridis(20),
        border_color="white",
        scale="none",
        cluster_rows=mash_asm_htree,
        cluster_cols=mash_asm_htree,
        silent=TRUE
    )
} else {
    TABS$mash <- NULL
    FIGS$mash <- NULL
}

##############################
# CD-HIT

if("cdhit" %in% snakemake@config$steps_analysis){
    TABS$cdhit <- read_cdhit(snakemake@input$cdhit[1])
    TABS$cdhit$tool1 <- factor(TABS$cdhit$tool1, ordered=TRUE, levels=ASM_TOOL_NAMES)
    TABS$cdhit$tool2 <- factor(TABS$cdhit$tool2, ordered=TRUE, levels=ASM_TOOL_NAMES)

    FIGS$cdhit <- list()

    FIGS$cdhit$unique                       <- plot_cdhit_barplots(df=TABS$cdhit, mtype="metaG", ycol="unique", ycol_text="Unique proteins [count]")
    FIGS$cdhit$unique_pct                   <- plot_cdhit_barplots(df=TABS$cdhit, mtype="metaG", ycol="unique_pct", ycol_text="Unique proteins [%]")
    FIGS$cdhit$highcovunique_pct_metag      <- plot_cdhit_barplots(df=TABS$cdhit, mtype="metaG", ycol="highcovunique_pct_unique", ycol_text="Unique proteins w/ high mean cov. [%, unique]")
    if("metaT" %in% TABS$cdhit$mtype){
        FIGS$cdhit$highcovunique_pct_metat  <- plot_cdhit_barplots(df=TABS$cdhit, mtype="metaT", ycol="highcovunique_pct_unique", ycol_text="Unique proteins w/ high mean cov. [%. unique]")
    }

    FIGS$cdhit$u_hcu_metag                  <- plot_cdhit_scatterplot(df=TABS$cdhit, mtype="metaG")
    if("metaT" %in% TABS$cdhit$mtype){
        FIGS$cdhit$u_hcu_metat              <- plot_cdhit_scatterplot(df=TABS$cdhit, mtype="metaT")
    }
} else {
    TABS$cdhit <- NULL
    FIGS$cdhit <- NULL
}
